@@ -1,6 +1,6 @@
 import typing
 import inspect
-from .model import CogCommandObject, CogSubcommandObject
+from .model import CogBaseCommandObject, CogSubcommandObject
 from .utils import manage_commands
 
 
@@ -9,6 +9,8 @@ def cog_slash(*,
               description: str = None,
               guild_ids: typing.List[int] = None,
               options: typing.List[dict] = None,
+              default_permission: bool = True,
+              permissions: typing.Dict[int, list] = None,
               connector: dict = None):
     """
     Decorator for Cog to add slash command.\n
@@ -34,6 +36,10 @@ def cog_slash(*,
     :type guild_ids: List[int]
     :param options: Options of the slash command. This will affect ``auto_convert`` and command data at Discord API. Default ``None``.
     :type options: List[dict]
+    :param default_permission: Sets if users have permission to run slash command by default, when no permissions are set. Default ``True``.
+    :type default_permission: bool
+    :param permissions: Dictionary of permissions of the slash command. Key being target guild_id and value being a list of permissions to apply. Default ``None``.
+    :type permissions: dict
     :param connector: Kwargs connector for the command. Default ``None``.
     :type connector: dict
     """
@@ -49,10 +55,12 @@ def cog_slash(*,
             "description": desc,
             "guild_ids": guild_ids,
             "api_options": opts,
+            "default_permission": default_permission,
+            "api_permissions": permissions,
             "connector": connector,
             "has_subcommands": False
         }
-        return CogCommandObject(name or cmd.__name__, _cmd)
+        return CogBaseCommandObject(name or cmd.__name__, _cmd)
     return wrapper
 
 
@@ -63,6 +71,8 @@ def cog_subcommand(*,
                    description: str = None,
                    base_description: str = None,
                    base_desc: str = None,
+                   base_default_permission: bool = True,
+                   base_permissions: typing.Dict[int, list] = None,
                    subcommand_group_description: str = None,
                    sub_group_desc: str = None,
                    guild_ids: typing.List[int] = None,
@@ -95,6 +105,10 @@ def cog_subcommand(*,
     :param base_description: Description of the base command. Default ``None``.
     :type base_description: str
     :param base_desc: Alias of ``base_description``.
+    :param base_default_permission: Sets if users have permission to run slash command by default, when no permissions are set. Default ``True``.
+    :type base_default_permission: bool
+    :param base_permissions: Dictionary of permissions of the slash command. Key being target guild_id and value being a list of permissions to apply. Default ``None``.
+    :type base_permissions: dict
     :param subcommand_group_description: Description of the subcommand_group. Default ``None``.
     :type subcommand_group_description: str
     :param sub_group_desc: Alias of ``subcommand_group_description``.
@@ -107,6 +121,7 @@ def cog_subcommand(*,
     """
     base_description = base_description or base_desc
     subcommand_group_description = subcommand_group_description or sub_group_desc
+    guild_ids = guild_ids if guild_ids else []
 
     def wrapper(cmd):
         desc = description or inspect.getdoc(cmd)
@@ -115,6 +130,17 @@ def cog_subcommand(*,
         else:
             opts = options
 
+        _cmd = {
+            "func": None,
+            "description": base_description,
+            "guild_ids": guild_ids.copy(),
+            "api_options": [],
+            "default_permission": base_default_permission,
+            "api_permissions": base_permissions,
+            "connector": {},
+            "has_subcommands": True
+        }
+
         _sub = {
             "func": cmd,
             "name": name or cmd.__name__,
@@ -125,5 +151,5 @@ def cog_subcommand(*,
             "api_options": opts,
             "connector": connector
         }
-        return CogSubcommandObject(_sub, base, name or cmd.__name__, subcommand_group)
+        return CogSubcommandObject(base, _cmd, subcommand_group, name or cmd.__name__, _sub)
     return wrapper
