@@ -72,6 +72,7 @@ class CommandData:
     :ivar description: Description of the command.
     :ivar options: List of :class:`OptionData`.
     :ivar id: Command id, this is received from discord so may not be present
+    :ivar application_id: The application id of the bot, required only when the application id and bot id are different. (old bots)
     """
 
     def __init__(
@@ -123,7 +124,6 @@ class CommandObject:
         self.allowed_guild_ids = cmd["guild_ids"] or []
         self.options = cmd["api_options"] or []
         self.connector = cmd["connector"] or {}
-        self.has_subcommands = cmd["has_subcommands"]
         # Ref https://github.com/Rapptz/discord.py/blob/master/discord/ext/commands/core.py#L1447
         # Since this isn't inherited from `discord.ext.commands.Command`, discord.py's check decorator will
         # add checks at this var.
@@ -131,18 +131,38 @@ class CommandObject:
         if hasattr(self.func, '__commands_checks__'):
             self.__commands_checks__ = self.func.__commands_checks__
 
-    async def invoke(self, *args, **kwargs):
+    async def invoke(self, *args):
         """
         Invokes the command.
 
         :param args: Args for the command.
         :raises: .error.CheckFailure
         """
-        can_run = await self.can_run(args[0])
+        args = list(args)
+        ctx = args.pop(0)
+        can_run = await self.can_run(ctx)
         if not can_run:
             raise error.CheckFailure
 
-        return await self.func(*args, **kwargs)
+        coro = None  # Get rid of annoying IDE complainings.
+
+        not_kwargs = False
+        if args and isinstance(args[0], dict):
+            kwargs = args[0]
+            ctx.kwargs = kwargs
+            ctx.args = list(kwargs.values())
+            try:
+                coro = self.func(ctx, **kwargs)
+            except TypeError:
+                args = list(kwargs.values())
+                not_kwargs = True
+        else:
+            ctx.args = args
+            not_kwargs = True
+        if not_kwargs:
+            coro = self.func(ctx, *args)
+
+        return await coro
 
     def add_check(self, func):
         """
@@ -176,6 +196,28 @@ class CommandObject:
         return False not in res
 
 
+class BaseCommandObject(CommandObject):
+    """
+    BaseCommand object of this extension.
+
+    .. note::
+        This model inherits :class:`.model.CommandObject`, so this has every variables from that.
+
+    .. warning::
+        Do not manually init this model.
+
+    :ivar has_subcommands: Indicates whether this base command has subcommands.
+    :ivar default_permission: Indicates whether users should have permissions to run this command by default.
+    :ivar permissions: Permissions to restrict use of this command.
+    """
+
+    def __init__(self, name, cmd):  # Let's reuse old command formatting.
+        super().__init__(name, cmd)
+        self.has_subcommands = cmd["has_subcommands"]
+        self.default_permission = cmd["default_permission"]
+        self.permissions = cmd["api_permissions"] or {}
+
+
 class SubcommandObject(CommandObject):
     """
     Subcommand object of this extension.
@@ -193,7 +235,6 @@ class SubcommandObject(CommandObject):
     """
 
     def __init__(self, sub, base, name, sub_group=None):
-        sub["has_subcommands"] = True  # For the inherited class.
         super().__init__(name, sub)
         self.base = base.lower()
         self.subcommand_group = sub_group.lower() if sub_group else sub_group
@@ -201,7 +242,7 @@ class SubcommandObject(CommandObject):
         self.subcommand_group_description = sub["sub_group_desc"]
 
 
-class CogCommandObject(CommandObject):
+class CogBaseCommandObject(BaseCommandObject):
     """
     Slash command object but for Cog.
 
@@ -220,11 +261,31 @@ class CogCommandObject(CommandObject):
         :param args: Args for the command.
         :raises: .error.CheckFailure
         """
-        can_run = await self.can_run(args[0])
+        args = list(args)
+        ctx = args.pop(0)
+        can_run = await self.can_run(ctx)
         if not can_run:
             raise error.CheckFailure
 
-        return await self.func(self.cog, *args, **kwargs)
+        coro = None  # Get rid of annoying IDE complainings.
+
+        not_kwargs = False
+        if args and isinstance(args[0], dict):
+            kwargs = args[0]
+            ctx.kwargs = kwargs
+            ctx.args = list(kwargs.values())
+            try:
+                coro = self.func(self.cog, ctx, **kwargs)
+            except TypeError:
+                args = list(kwargs.values())
+                not_kwargs = True
+        else:
+            ctx.args = args
+            not_kwargs = True
+        if not_kwargs:
+            coro = self.func(self.cog, ctx, *args)
+
+        return await coro
 
 
 class CogSubcommandObject(SubcommandObject):
@@ -235,8 +296,9 @@ class CogSubcommandObject(SubcommandObject):
         Do not manually init this model.
     """
 
-    def __init__(self, *args):
-        super().__init__(*args)
+    def __init__(self, base, cmd, sub_group, name, sub):
+        super().__init__(sub, base, name, sub_group)
+        self.base_command_data = cmd
         self.cog = None  # Manually set this later.
 
     async def invoke(self, *args, **kwargs):
@@ -246,11 +308,31 @@ class CogSubcommandObject(SubcommandObject):
         :param args: Args for the command.
         :raises: .error.CheckFailure
         """
-        can_run = await self.can_run(args[0])
+        args = list(args)
+        ctx = args.pop(0)
+        can_run = await self.can_run(ctx)
         if not can_run:
             raise error.CheckFailure
 
-        return await self.func(self.cog, *args, **kwargs)
+        coro = None  # Get rid of annoying IDE complainings.
+
+        not_kwargs = False
+        if args and isinstance(args[0], dict):
+            kwargs = args[0]
+            ctx.kwargs = kwargs
+            ctx.args = list(kwargs.values())
+            try:
+                coro = self.func(self.cog, ctx, **kwargs)
+            except TypeError:
+                args = list(kwargs.values())
+                not_kwargs = True
+        else:
+            ctx.args = args
+            not_kwargs = True
+        if not_kwargs:
+            coro = self.func(self.cog, ctx, *args)
+
+        return await coro
 
 
 class SlashCommandOptionType(IntEnum):
@@ -336,7 +418,7 @@ class SlashMessage(discord.Message):
 
     async def edit(self, **fields):
         """Refer :meth:`discord.Message.edit`."""
-        if "file" in fields or "files" in fields:
+        if "file" in fields or "files" in fields or "embeds" in fields:
             await self._slash_edit(**fields)
         else:
             try:
@@ -358,3 +440,67 @@ class SlashMessage(discord.Message):
                     await self._http.delete(self.__interaction_token, self.id)
 
             self._state.loop.create_task(wrap())
+
+
+class PermissionData:
+    """
+    Single slash permission data.
+
+    :ivar id: User or role id, based on following type specfic.
+    :ivar type: The ``SlashCommandPermissionsType`` type of this permission.
+    :ivar permission: State of permission. ``True`` to allow, ``False`` to disallow.
+    """
+    def __init__(self, id, type, permission, **kwargs):
+        self.id = id
+        self.type = type
+        self.permission = permission
+
+    def __eq__(self, other):
+        if isinstance(other, PermissionData):
+            return (
+                self.id == other.id
+                and self.type == other.id
+                and self.permission == other.permission
+            )
+        else:
+            return False
+
+
+class GuildPermissionsData:
+    """
+    Slash permissions data for a command in a guild.
+
+    :ivar id: Command id, provided by discord.
+    :ivar guild_id: Guild id that the permissions are in. 
+    :ivar permissions: List of permissions dict.
+    """
+    def __init__(self, id, guild_id, permissions, **kwargs):
+        self.id = id
+        self.guild_id = guild_id
+        self.permissions = []
+        if permissions:
+            for permission in permissions:
+                self.permissions.append(PermissionData(**permission))
+
+    def __eq__(self, other):
+        if isinstance(other, GuildPermissionsData):
+            return (
+                self.id == other.id
+                and self.guild_id == other.guild_id
+                and self.permissions == other.permissions
+            )
+        else:
+            return False
+
+
+class SlashCommandPermissionType(IntEnum):
+    """
+    Equivalent of `ApplicationCommandPermissionType <https://discord.com/developers/docs/interactions/slash-commands#applicationcommandpermissiontype>`_  in the Discord API.
+    """
+    ROLE = 1
+    USER = 2
+
+    @classmethod
+    def from_type(cls, t: type):
+        if issubclass(t, discord.abc.Role): return cls.ROLE
+        if issubclass(t, discord.abc.User): return cls.USER
